@@ -1,6 +1,6 @@
 '''
-Structure and API
-=================
+Structure
+=========
 
 The NDB objects are dictionary-like structures that represent network
 objects -- interfaces, routes, addresses etc. In addition to the
@@ -80,6 +80,9 @@ not committed changes. To inspect cached objects, use views' `.cache`::
 
 There is no asynchronous cache invalidation, the cache is being cleaned up
 every time when an object is accessed.
+
+API
+===
 '''
 import json
 import time
@@ -92,31 +95,60 @@ from functools import partial
 from pyroute2 import cli
 from pyroute2.ndb.events import State
 from pyroute2.ndb.report import Record
+from pyroute2.ndb.auth_manager import check_auth
+from pyroute2.ndb.auth_manager import AuthManager
 from pyroute2.netlink.exceptions import NetlinkError
 from pyroute2.ndb.events import InvalidateHandlerException
 
+RSLV_IGNORE = 0
+RSLV_RAISE = 1
+RSLV_NONE = 2
+RSLV_DELETE = 3
+
+
+class Spec(object):
+    '''
+    A universal NDB object spec
+    '''
+    def __init__(self, iclass, spec):
+        if isinstance(spec, Record):
+            spec = spec._as_dict()
+        self.spec = spec
+        self.iclass = iclass
+        self.normalize()
+
+    def normalize(self):
+        self.spec = self.iclass.spec_normalize(self.spec)
+        return self
+
+    def load_context(self, context):
+        self.spec.update(context)
+        return self
+
+    @property
+    def get_spec(self):
+        return self.spec
+
 
 class RTNL_Object(dict):
     '''
-    Base RTNL object class.
+    The common base class for NDB objects -- interfaces, routes, rules
+    addresses etc.
     '''
 
     view = None    # (optional) view to load values for the summary etc.
     utable = None  # table to send updates to
-    table_alias = ''
 
+    resolve_fields = []
     key_extra_fields = []
     hidden_fields = []
     fields_cmp = {}
+    fields_normalize = {}
 
     schema = None
     event_map = None
     state = None
     log = None
-    summary = None
-    summary_header = None
-    dump = None
-    dump_header = None
     errors = None
     msg_class = None
     reverse_update = None
@@ -125,6 +157,7 @@ class RTNL_Object(dict):
     _key = None
     _replace = None
     _replace_on_key_change = False
+    _init_complete = False
 
     # 8<------------------------------------------------------------
     #
@@ -133,8 +166,10 @@ class RTNL_Object(dict):
     @property
     def table(self):
         '''
-        The main reference table for the object. The SQL schema of the
-        table is used to build the key and to verify the fields.
+        Main reference table for the object. The SQL schema of this
+        table is used to build the object key and to verify fields.
+
+        Read-write property.
         '''
         return self._table
 
@@ -145,8 +180,8 @@ class RTNL_Object(dict):
     @property
     def etable(self):
         '''
-        The table where the object actually fetches the data from. It
-        is not always equal `self.table`, e.g. snapshot objects fetch
+        Effective table where the object actually fetches the data from.
+        It is not always equal `self.table`, e.g. snapshot objects fetch
         the data from snapshot tables.
 
         Read-only property.
@@ -159,7 +194,10 @@ class RTNL_Object(dict):
     @property
     def key(self):
         '''
-        The key of the object, used to fetch it from the DB.
+        Key of the object, used to build SQL requests to fetch
+        the data from the DB.
+
+        Read-write property.
         '''
         nkey = self._key or {}
         ret = collections.OrderedDict()
@@ -169,6 +207,8 @@ class RTNL_Object(dict):
                 value = self[kname]
                 if value is None and name in nkey:
                     value = nkey[name]
+                if isinstance(value, (list, tuple, dict)):
+                    value = json.dumps(value)
                 ret[name] = value
         if len(ret) < len(self.kspec):
             for name in self.key_extra_fields:
@@ -189,21 +229,57 @@ class RTNL_Object(dict):
     # 8<------------------------------------------------------------
     #
 
+    @classmethod
+    def _dump_where(cls, view):
+        return '', []
+
+    @classmethod
+    def _sdump(cls, view, names, fnames):
+        req = '''
+              SELECT %s FROM %s AS main
+              ''' % (fnames, cls.table)
+        yield names
+        where, values = cls._dump_where(view)
+        for record in view.ndb.schema.fetch(req + where, values):
+            yield record
+
+    @classmethod
+    def summary(cls, view):
+        return cls._sdump(view,
+                          view.ndb.schema.compiled[cls.table]['norm_idx'],
+                          view.ndb.schema.compiled[cls.table]['knames'])
+
+    @classmethod
+    def dump(cls, view):
+        return cls._sdump(view,
+                          view.ndb.schema.compiled[cls.table]['norm_names'],
+                          view.ndb.schema.compiled[cls.table]['fnames'])
+
+    @staticmethod
+    def spec_normalize(spec):
+        if 'target' not in spec:
+            spec['target'] = 'localhost'
+        return spec
+
+    @staticmethod
+    def key_load_context(key, context):
+        return key
+
     def __init__(self,
                  view,
                  key,
                  iclass,
                  ctxid=None,
-                 match_src=None,
-                 match_pairs=None,
-                 load=True):
+                 load=True,
+                 master=None,
+                 check=True,
+                 auth_managers=None):
         self.view = view
         self.ndb = view.ndb
         self.sources = view.ndb.sources
+        self.master = master
         self.ctxid = ctxid
         self.schema = view.ndb.schema
-        self.match_src = match_src or tuple()
-        self.match_pairs = match_pairs or dict()
         self.changed = set()
         self.iclass = iclass
         self.utable = self.utable or self.table
@@ -211,16 +287,23 @@ class RTNL_Object(dict):
         self.atime = time.time()
         self.log = self.ndb.log.channel('rtnl_object')
         self.log.debug('init')
+        if auth_managers is None:
+            auth_managers = [AuthManager(None, self.ndb.log.channel('auth'))]
+        self.auth_managers = auth_managers
         self.state = State()
         self.state.set('invalid')
         self.snapshot_deps = []
         self.load_event = threading.Event()
         self.load_event.set()
+        self.load_debug = False
         self.lock = threading.Lock()
         self.kspec = self.schema.compiled[self.table]['idx']
         self.knorm = self.schema.compiled[self.table]['norm_idx']
         self.spec = self.schema.compiled[self.table]['all_names']
         self.names = self.schema.compiled[self.table]['norm_names']
+        self.last_save = None
+        if self.event_map is None:
+            self.event_map = {}
         self._apply_script = []
         if isinstance(key, dict):
             self.chain = key.pop('ndb_chain', None)
@@ -228,17 +311,18 @@ class RTNL_Object(dict):
         else:
             self.chain = None
             create = False
+        exists = self.exists(key)
         ckey = self.complete_key(key)
         if create:
-            if ckey is not None:
+            if check & exists:
                 raise KeyError('object exists')
             for name in key:
-                self[name] = key[name]
+                self[self.iclass.nla2name(name)] = key[name]
             # FIXME -- merge with complete_key()
             if 'target' not in self:
                 self.load_value('target', self.view.default_target)
         else:
-            if ckey is None:
+            if not exists:
                 raise KeyError('object does not exists')
             self.key = ckey
             if load:
@@ -246,6 +330,35 @@ class RTNL_Object(dict):
                     self.load_sql()
                 else:
                     self.load_sql(table=self.table)
+        self._init_complete = True
+
+    @classmethod
+    def new_spec(cls, spec):
+        return Spec(cls, spec)
+
+    @staticmethod
+    def resolve(view, spec, fields, policy=RSLV_IGNORE):
+        '''
+        Resolve specific fields e.g. convert port ifname into index::
+        '''
+        for field in fields:
+            ref = spec.get(field)
+            if ref is not None and not isinstance(ref, int):
+                try:
+                    spec[field] = view[ref]['index']
+                except KeyError:
+                    if policy == RSLV_RAISE:
+                        raise
+                    elif policy == RSLV_NONE:
+                        spec[field] = None
+                    elif policy == RSLV_DELETE:
+                        del spec[field]
+                    elif policy == RSLV_IGNORE:
+                        pass
+                    else:
+                        raise TypeError('unknown rslv policy')
+
+        return spec
 
     def mark_tflags(self, mark):
         pass
@@ -258,22 +371,22 @@ class RTNL_Object(dict):
         return filter(lambda x: x[0] not in self.hidden_fields,
                       dict.items(self))
 
-    @classmethod
-    def adjust_spec(cls, spec, context):
-        return spec
+    @property
+    def context(self):
+        return {'target': self.get('target', 'localhost')}
 
     @classmethod
     def nla2name(self, name):
         return self.msg_class.nla2name(name)
 
+    @classmethod
+    def compare_record(self, left, right):
+        pass
+
     @classmethod
     def name2nla(self, name):
         return self.msg_class.name2nla(name)
 
-    @property
-    def context(self):
-        return {'target': self.get('target', 'localhost')}
-
     def __enter__(self):
         return self
 
@@ -283,20 +396,21 @@ class RTNL_Object(dict):
     def __hash__(self):
         return id(self)
 
+    @check_auth('obj:read')
     def __getitem__(self, key):
-        if key in self.match_pairs:
-            for src in self.match_src:
-                try:
-                    return src[self.match_pairs[key]]
-                except:
-                    pass
         return dict.__getitem__(self, key)
 
+    @check_auth('obj:modify')
     def __setitem__(self, key, value):
+        if key in self.fields_normalize:
+            value = self.fields_normalize[key](value)
+
         if self.state == 'system' and key in self.knorm:
             if self._replace_on_key_change:
                 self.log.debug('prepare replace for key %s' % (self.key))
-                self._replace = type(self)(self.view, self.key)
+                self._replace = type(self)(self.view,
+                                           self.key,
+                                           auth_managers=self.auth_managers)
                 self.state.set('replace')
             else:
                 raise ValueError('attempt to change a key field (%s)' % key)
@@ -321,6 +435,7 @@ class RTNL_Object(dict):
         return self.view[spec]
 
     @cli.show_result
+    @check_auth('obj:read')
     def show(self, **kwarg):
         '''
         Return the object in a specified format. The format may be
@@ -341,7 +456,7 @@ class RTNL_Object(dict):
                 out[key] = self[key]
             return '%s\n' % json.dumps(out, indent=4, separators=(',', ': '))
 
-    def set(self, key, value):
+    def set(self, *argv, **kwarg):
         '''
         Set a field specified by `key` to `value`, and return self. The
         method is useful to write call chains like that::
@@ -353,6 +468,12 @@ class RTNL_Object(dict):
              .set('address', '00:11:22:33:44:55')
              .commit())
         '''
+        key, value = None, None
+        if argv:
+            key, value = argv
+        elif kwarg:
+            for key, value in kwarg.items():
+                break
         self[key] = value
         return self
 
@@ -388,6 +509,7 @@ class RTNL_Object(dict):
              .register_handler(event,
                                partial(wr_handler, wr, fname)))
 
+    @check_auth('obj:modify')
     def snapshot(self, ctxid=None):
         '''
         Create and return a snapshot of the object. The method creates
@@ -402,7 +524,10 @@ class RTNL_Object(dict):
             key = self.key
         else:
             key = self._replace.key
-        snp = type(self)(self.view, key, ctxid=ctxid)
+        snp = type(self)(self.view,
+                         key,
+                         ctxid=ctxid,
+                         auth_managers=self.auth_managers)
         self.ndb.schema.save_deps(ctxid, weakref.ref(snp), self.iclass)
         snp.changed = set(self.changed)
         return snp
@@ -424,6 +549,14 @@ class RTNL_Object(dict):
         fetch = []
         if isinstance(key, Record):
             key = key._as_dict()
+        else:
+            key = dict(key)
+
+        self.resolve(view=self.view,
+                     spec=key,
+                     fields=self.resolve_fields,
+                     policy=RSLV_DELETE)
+
         for name in self.kspec:
             if name not in key:
                 fetch.append('f_%s' % name)
@@ -455,6 +588,13 @@ class RTNL_Object(dict):
         self.log.debug('got %s' % key)
         return key
 
+    def exists(self, key):
+        '''
+        Check if the object exists in the DB
+        '''
+        return self.view.exists(key)
+
+    @check_auth('obj:modify')
     def rollback(self, snapshot=None):
         '''
         Try to rollback the object state using the snapshot provided as
@@ -463,18 +603,23 @@ class RTNL_Object(dict):
         if self._replace is not None:
             self.log.debug('rollback replace: %s :: %s'
                            % (self.key, self._replace.key))
-            new_replace = type(self)(self.view, self.key)
+            new_replace = type(self)(self.view,
+                                     self.key,
+                                     auth_managers=self.auth_managers)
             new_replace.state.set('remove')
             self.state.set('replace')
             self.update(self._replace)
             self._replace = new_replace
         self.log.debug('rollback: %s' % str(self.state.events))
         snapshot = snapshot or self.last_save
-        snapshot.state.set(self.state.get())
-        snapshot.apply(rollback=True)
-        for link, snp in snapshot.snapshot_deps:
-            link.rollback(snapshot=snp)
-        return self
+        if snapshot == -1:
+            return self.remove().apply()
+        else:
+            snapshot.state.set(self.state.get())
+            snapshot.apply(rollback=True)
+            for link, snp in snapshot.snapshot_deps:
+                link.rollback(snapshot=snp)
+            return self
 
     def clear(self):
         pass
@@ -485,6 +630,7 @@ class RTNL_Object(dict):
             not self.changed and \
             not self._apply_script
 
+    @check_auth('obj:modify')
     def commit(self):
         '''
         Try to commit the pending changes. If the commit fails,
@@ -538,10 +684,11 @@ class RTNL_Object(dict):
                 e_r.chain = None
             raise
         finally:
-            (self
-             .last_save
-             .state
-             .set(self.state.get()))
+            if self.last_save is not None:
+                (self
+                 .last_save
+                 .state
+                 .set(self.state.get()))
         if self._replace is not None:
             self._replace = None
         return self
@@ -578,6 +725,12 @@ class RTNL_Object(dict):
             req[key] = self[key]
         return req
 
+    def make_idx_req(self, prime):
+        return prime
+
+    def sync_req(self, prime):
+        return prime
+
     def get_count(self):
         conditions = []
         values = []
@@ -596,15 +749,25 @@ class RTNL_Object(dict):
     def hook_apply(self, method, **spec):
         pass
 
-    def apply(self, rollback=False):
+    @check_auth('obj:modify')
+    def apply(self, rollback=False, req_filter=None):
         '''
         Create a snapshot and apply pending changes. Do not revert
         the changes in the case of an exception.
         '''
 
+        # Resolve the fields
+        self.resolve(view=self.view,
+                     spec=self,
+                     fields=self.resolve_fields,
+                     policy=RSLV_RAISE)
+
         # Save the context
-        if not rollback and self.state != 'invalid':
-            self.last_save = self.snapshot()
+        if not rollback:
+            if self.state == 'invalid':
+                self.last_save = -1
+            else:
+                self.last_save = self.snapshot()
 
         self.log.debug('apply: %s' % str(self.state.events))
         self.load_event.clear()
@@ -625,7 +788,8 @@ class RTNL_Object(dict):
         idx_req = dict([(x, self[self.iclass.nla2name(x)]) for x
                         in self.schema.compiled[self.table]['idx']
                         if self.iclass.nla2name(x) in self])
-        req = self.make_req(idx_req)
+        req = self.sync_req(self.make_req(idx_req))
+        idx_req = self.make_idx_req(idx_req)
         self.log.debug('apply req: %s' % str(req))
         self.log.debug('apply idx_req: %s' % str(idx_req))
         self.log.debug('apply state: %s' % state)
@@ -634,16 +798,18 @@ class RTNL_Object(dict):
         ignore = tuple()
         #
         if state in ('invalid', 'replace'):
-            req = dict([x for x in self.items() if x[1] is not None])
-            for l_key, r_key in self.match_pairs.items():
-                for src in self.match_src:
-                    try:
-                        req[l_key] = src[r_key]
-                        break
-                    except:
-                        pass
+            for k, v in tuple(self.items()):
+                if k not in req and v is not None:
+                    req[k] = v
+            if self.master is not None:
+                req = (self
+                       .new_spec(req)
+                       .load_context(self.master.context)
+                       .get_spec)
+
             method = 'add'
-            ignore = {errno.EEXIST: 'set'}
+            ignore = {errno.EEXIST: 'set',
+                      errno.EAGAIN: None}
         elif state == 'system':
             method = 'set'
         elif state == 'setns':
@@ -653,11 +819,15 @@ class RTNL_Object(dict):
             method = 'del'
             req = idx_req
             ignore = {errno.ENODEV: None,         # interfaces
+                      errno.ENOENT: None,         # rules
                       errno.ESRCH: None,          # routes
                       errno.EADDRNOTAVAIL: None}  # addresses
         else:
             raise Exception('state transition not supported')
 
+        if req_filter is not None:
+            req = req_filter(req)
+
         for itn in range(20):
             try:
                 self.log.debug('run %s (%s)' % (method, req))
@@ -670,6 +840,21 @@ class RTNL_Object(dict):
                 (self
                  .log
                  .debug('error: %s' % e))
+                ##
+                #
+                # FIXME: performance penalty
+                # required now only in some NDA corner cases
+                # must be moved to objects.neighbour
+                #
+                if e.code == errno.EEXIST:
+                    update = (self
+                              .sources[self['target']]
+                              .api(self.api, 'dump'))
+                    update = list(update)
+                    self.ndb._event_queue.put(update)
+                    self.load_sql()
+                #
+                ##
                 if e.code in ignore:
                     self.log.debug('ignore error %s for %s' % (e.code, self))
                     if ignore[e.code] is not None:
@@ -714,10 +899,11 @@ class RTNL_Object(dict):
         if rollback:
             #
             # Iterate all the snapshot tables and collect the diff
-            for (table, cls) in self.view.classes.items():
+            for cls in self.view.classes.values():
                 if issubclass(type(self), cls) or \
                         issubclass(cls, type(self)):
                     continue
+                table = cls.table
                 # comprare the tables
                 diff = (self
                         .ndb
@@ -735,9 +921,14 @@ class RTNL_Object(dict):
                                       record))
                     key = dict([x for x in record.items()
                                 if x[0] in self.schema.compiled[table]['idx']])
-                    obj = self.view.get(key, table)
+                    key['create'] = True
+                    try:
+                        obj = self.view.template(key, table)
+                    except KeyError:
+                        continue
                     obj.load_sql(ctxid=self.ctxid)
                     obj.state.set('invalid')
+                    obj.register()
                     try:
                         obj.apply()
                     except Exception as e:
@@ -761,12 +952,18 @@ class RTNL_Object(dict):
         Load a value and clean up the `self.changed` set if the
         loaded value matches the expectation.
         '''
+        if self.load_debug:
+            self.log.debug('load %s: %s' % (key, value))
+
         if key not in self.changed:
             dict.__setitem__(self, key, value)
         elif self.get(key) == value:
             self.changed.remove(key)
         elif key in self.fields_cmp and self.fields_cmp[key](self, value):
             self.changed.remove(key)
+        elif self.load_debug:
+            self.log.debug('discard %s: %s (expected %s)'
+                           % (key, value, self.get(key)))
 
     def load_sql(self, table=None, ctxid=None, set_state=True):
         '''
@@ -786,6 +983,8 @@ class RTNL_Object(dict):
 
         for name, value in self.key.items():
             keys.append('f_%s = %s' % (name, self.schema.plch))
+            if isinstance(value, (list, tuple, dict)):
+                value = json.dumps(value)
             values.append(value)
 
         spec = (self
