@@ -88,6 +88,7 @@ There are also some useful views, that join `ifinfo` tables with
     (8 rows)
 
 '''
+import io
 import sys
 import time
 import random
@@ -161,7 +162,7 @@ def publish(method):
         else:
             # another thread, run via message bus
             self._allow_read.wait()
-            response = queue.Queue(maxsize=4096)
+            response = queue.Queue()
             request = cmsg_req(response, *argv, **kwarg)
             self.ndb._event_queue.put((request, ))
             while True:
@@ -295,14 +296,6 @@ class DBSchema(object):
                                                   self.indices[table]))
             self.create_table(table)
 
-            if table.startswith('ifinfo_'):
-                idx = ('index', )
-                self.compiled[table[7:]] = self.merge_spec('interfaces',
-                                                           table,
-                                                           table[7:],
-                                                           idx)
-                self.create_ifinfo_view(table)
-
         #
         # service tables
         #
@@ -528,11 +521,13 @@ class DBSchema(object):
                 yield row
 
     @publish_exec
-    def export(self, fname='stdout'):
-        if fname in ('stdout', 'stderr'):
-            f = getattr(sys, fname)
-        else:
-            f = open(fname, 'w')
+    def export(self, f='stdout'):
+        close = False
+        if f in ('stdout', 'stderr'):
+            f = getattr(sys, f)
+        elif isinstance(f, basestring):
+            f = open(f, 'w')
+            close = True
         try:
             for table in self.spec.keys():
                 f.write('\ntable %s\n' % table)
@@ -547,9 +542,14 @@ class DBSchema(object):
                         f.write(' '.join([str(x) for x in record]))
                         f.write('\n')
         finally:
-            if fname not in ('stdout', 'stderr'):
+            if close:
                 f.close()
 
+    def __repr__(self):
+        buf = io.StringIO()
+        self.export(buf)
+        return buf.getvalue()
+
     @publish_exec
     def close(self):
         self.purge_snapshots()
@@ -560,31 +560,6 @@ class DBSchema(object):
     def commit(self):
         self.connection.commit()
 
-    def create_ifinfo_view(self, table, ctxid=None):
-        # FIXME: move to the object module
-        iftable = 'interfaces'
-
-        req = (('main.f_target', 'main.f_tflags') +
-               tuple(['main.f_%s' % x[-1] for x
-                      in self.spec['interfaces'].keys()]) +
-               tuple(['data.f_%s' % x[-1] for x
-                      in self.spec[table].keys()])[:-2])
-        # -> ... main.f_index, main.f_IFLA_IFNAME, ..., data.f_IFLA_BR_GC_TIMER
-        if ctxid is not None:
-            iftable = '%s_%s' % (iftable, ctxid)
-            table = '%s_%s' % (table, ctxid)
-        self.execute('''
-                     DROP VIEW IF EXISTS %s
-                     ''' % table[7:])
-        self.execute('''
-                     CREATE VIEW %s AS
-                     SELECT %s FROM %s AS main
-                     INNER JOIN %s AS data ON
-                         main.f_index = data.f_index
-                     AND
-                         main.f_target = data.f_target
-                     ''' % (table[7:], ','.join(req), iftable, table))
-
     def create_table(self, table):
         req = ['f_target TEXT NOT NULL',
                'f_tflags BIGINT NOT NULL DEFAULT 0']
@@ -733,9 +708,6 @@ class DBSchema(object):
                          '''
                          % (table, ctxid, table, self.plch),
                          [uuid])
-
-            if table.startswith('ifinfo_'):
-                self.create_ifinfo_view(table, ctxid)
         #
         # unmark all the data
         #
@@ -816,7 +788,7 @@ class DBSchema(object):
         self.execute('INSERT INTO %s_log (%s) VALUES (%s)'
                      % (table, fields, pch), values)
 
-    def load_netlink(self, table, target, event, ctable=None):
+    def load_netlink(self, table, target, event, ctable=None, propagate=False):
         #
         if self.rtnl_log:
             self.log_netlink(table, target, event, ctable)
@@ -953,7 +925,10 @@ class DBSchema(object):
                 else:
                     raise NotImplementedError()
                 #
-            except Exception:
+            except Exception as e:
+                #
+                if propagate:
+                    raise e
                 #
                 # A good question, what should we do here
                 self.log.debug('load_netlink: %s %s %s'
@@ -971,22 +946,14 @@ def init(ndb, connection, mode, rtnl_log, tid):
         # 1. spec
         #
         for name, spec in plugin.init['specs']:
-            DBSchema.spec[name] = spec
+            DBSchema.spec[name] = spec.as_dict()
+            DBSchema.indices[name] = spec.index
+            DBSchema.foreign_keys[name] = spec.foreign_keys
         #
         # 2. classes
         #
         for name, cls in plugin.init['classes']:
             DBSchema.classes[name] = cls
-        #
-        # 3. indices
-        #
-        for name, idx in plugin.init['indices']:
-            DBSchema.indices[name] = idx
-        #
-        # 4. foreign keys
-        #
-        for name, kspec in plugin.init['foreign_keys']:
-            DBSchema.foreign_keys[name] = kspec
 
     ret = DBSchema(ndb, connection, mode, rtnl_log, tid)
 
