@@ -1,6 +1,4 @@
 '''
-RTNL sources
-============
 
 Local RTNL
 ----------
@@ -72,23 +70,24 @@ See also: :ref:`remote`
 '''
 import sys
 import time
+import uuid
+import queue
 import errno
 import socket
 import struct
 import threading
 from pyroute2 import IPRoute
 from pyroute2 import RemoteIPRoute
-from pyroute2.ndb.events import (SchemaReadLock,
-                                 SchemaReadUnlock,
-                                 ShutdownException,
+from pyroute2.ndb.events import (ShutdownException,
                                  State)
-from pyroute2.ndb.messages import (cmsg,
-                                   cmsg_event,
+from pyroute2.ndb.messages import (cmsg_event,
                                    cmsg_failed,
                                    cmsg_sstart)
 from pyroute2.netlink.nlsocket import NetlinkMixin
 from pyroute2.netlink.exceptions import NetlinkError
+from pyroute2.netlink.rtnl.ifinfmsg import ifinfmsg
 if sys.platform.startswith('linux'):
+    from pyroute2 import netns
     from pyroute2.netns.nslink import NetNS
     from pyroute2.netns.manager import NetNSManager
 else:
@@ -98,6 +97,35 @@ else:
 SOURCE_FAIL_PAUSE = 5
 
 
+class SourceProxy(object):
+
+    def __init__(self, ndb, target):
+        self.ndb = ndb
+        self.events = queue.Queue()
+        self.target = target
+
+    def api(self, name, *argv, **kwarg):
+        call_id = str(uuid.uuid4().hex)
+        self.ndb._call_registry[call_id] = event = threading.Event()
+        event.clear()
+        (self
+         .ndb
+         .messenger
+         .emit({'type': 'api',
+                'target': self.target,
+                'call_id': call_id,
+                'name': name,
+                'argv': argv,
+                'kwarg': kwarg}))
+
+        event.wait()
+        response = self.ndb._call_registry.pop(call_id)
+        if 'return' in response:
+            return response['return']
+        elif 'exception' in response:
+            raise response['exception']
+
+
 class Source(dict):
     '''
     The RNTL source. The source that is used to init the object
@@ -131,10 +159,13 @@ class Source(dict):
         self.nl_prime = self.vmap[self.kind]
         self.nl_kwarg = spec
         #
+        if self.ndb.messenger is not None:
+            self.ndb.messenger.targets.add(self.target)
+        #
         self.shutdown = threading.Event()
         self.started = threading.Event()
         self.lock = threading.RLock()
-        self.shutdown_lock = threading.Lock()
+        self.shutdown_lock = threading.RLock()
         self.started.clear()
         self.log = ndb.log.channel('sources.%s' % self.target)
         self.state = State(log=self.log)
@@ -170,6 +201,7 @@ class Source(dict):
         if 'netns' in spec:
             defaults['kind'] = 'netns'
             defaults['target'] = spec['netns']
+            ret['netns'] = netns._get_netnspath(spec['netns'])
         for key in defaults:
             if key not in ret:
                 ret[key] = defaults[key]
@@ -191,6 +223,12 @@ class Source(dict):
     def name2nla(cls, name):
         return name
 
+    @classmethod
+    def summary(cls, view):
+        yield ('state', 'name', 'spec')
+        for key in view.keys():
+            yield (view[key].state.get(), key, '%s' % (view[key].nl_kwarg, ))
+
     def api(self, name, *argv, **kwarg):
         for _ in range(100):  # FIXME make a constant
             with self.lock:
@@ -210,6 +248,20 @@ class Source(dict):
                     time.sleep(1)
         raise RuntimeError('api call failed')
 
+    def fake_zero_if(self):
+        url = 'https://github.com/svinota/pyroute2/issues/737'
+        zero_if = ifinfmsg()
+        zero_if['index'] = 0
+        zero_if['state'] = 'up'
+        zero_if['flags'] = 1
+        zero_if['header']['flags'] = 2
+        zero_if['header']['type'] = 16
+        zero_if['header']['target'] = self.target
+        zero_if['event'] = 'RTM_NEWLINK'
+        zero_if['attrs'] = [('IFLA_IFNAME', url),
+                            ('IFLA_ADDRESS', '00:00:00:00:00:00')]
+        self.evq.put([zero_if, ])
+
     def receiver(self):
         #
         # The source thread routine -- get events from the
@@ -247,6 +299,8 @@ class Source(dict):
                     self.ndb.schema.allow_read(False)
                     try:
                         self.ndb.schema.flush(self.target)
+                        if self.kind in ('local', 'netns', 'remote'):
+                            self.fake_zero_if()
                         self.evq.put(self.nl.dump())
                     finally:
                         self.ndb.schema.allow_read(True)
@@ -341,7 +395,7 @@ class Source(dict):
             if self.shutdown.is_set():
                 self.log.debug('already stopped')
                 return
-            self.log.info('source shutdown')
+            self.log.debug('source shutdown')
             self.shutdown.set()
             if self.nl is not None:
                 try:
@@ -357,16 +411,19 @@ class Source(dict):
 
     def restart(self, reason='unknown'):
         with self.lock:
-            if not self.shutdown.is_set():
+            with self.shutdown_lock:
                 self.log.debug('restarting the source, reason <%s>' % (reason))
-                self.evq.put((cmsg(self.target, SchemaReadLock()), ))
+                self.started.clear()
+                self.ndb.schema.allow_read(False)
                 try:
                     self.close()
                     if self.th:
                         self.th.join()
+                    self.shutdown.clear()
                     self.start()
                 finally:
-                    self.evq.put((cmsg(self.target, SchemaReadUnlock()), ))
+                    self.ndb.schema.allow_read(True)
+        self.started.wait()
 
     def __enter__(self):
         return self
