@@ -30,8 +30,20 @@ NFNL_SUBSYS_NFTABLES = 10
 NFNL_SUBSYS_NFT_COMPAT = 11
 NFNL_SUBSYS_COUNT = 12
 
+# multicast group ids (for use with {add,drop}_membership)
+NFNLGRP_NONE = 0
+NFNLGRP_CONNTRACK_NEW = 1
+NFNLGRP_CONNTRACK_UPDATE = 2
+NFNLGRP_CONNTRACK_DESTROY = 3
+NFNLGRP_CONNTRACK_EXP_NEW = 4
+NFNLGRP_CONNTRACK_EXP_UPDATE = 5
+NFNLGRP_CONNTRACK_EXP_DESTROY = 6
+NFNLGRP_NFTABLES = 7
+NFNLGRP_ACCT_QUOTA = 8
+NFNLGRP_NFTRACE = 9
+
 
 class nfgen_msg(nlmsg):
     fields = (('nfgen_family', 'B'),
               ('version', 'B'),
-              ('res_id', 'H'))
+              ('res_id', '!H'))
